#! /usr/bin/perl
################################################################################
# mcg: mail configuration manager version 0.3 20030130
# Script to create procmail and mutt folder rules for a series of folders.
# folders are also linked to mutt profiles automatically. For the concept of
# mutt profiles, see:
#         http://www.acoustics.hut.fi/~mara/mutt/muttprofile.html
# and     http://www.ratblast.net/muttprofile/
#
# useful to give to folders names unrelated to list headers, propagating any
# change both to .procmailrc and .muttrc
#
# Distributed under GNU General Public License
# Copyright 2003 Marco Fioretti marco@rule-project.org
################################################################################
#
# CHANGELOG:
#
# version 0.3
#
# 1) to_hdr feature better documented (see below)
#
# 2) trailing and leading spaces removed from $MEMBER addresses in __GROUP__
#    recipes (they would generated errors when sourcing the corresponding
#    send-hook from .muttrc
# 3) Line which added default "send-hook ." to .muttrc commented because
#   send-hook overrides previous folder hooks, i.e. a folder hook loads a
#   profile, but "send-hook . default seems (at least in my mutt version
#   (Mutt 1.4i (2002-05-29)) to reload the default profile when replying from
#   that folder

# version 0.2
# (many thanks to Brian Salter-Duke for patches, testing and suggestions on 
#  practically all the improvements listed below)
#
#
# 1) $YEAR substituted by calls to /bin/date (change it if the path to date 
#    is different on your system!! The reason is that mcg generates rules that 
#    mutt and procmail could use even years later so it must leave to them the
#    calculation of the current year/month. In other words, version 0.1 was not
#    "Year-2004-compatible" :-)
#
# 2) "lists" and "subscribe" are mutually exclusive in mutt
#    Now there is only one list directive, "lists:" which can be
#           missing entirely (no list)
#           'S'           (will add "subscribe" line to $MUTT_RULES
#           'L'           (will add "lists"     line to $MUTT_RULES
#
# 3) "save:" directive added, to customize save-hook. Currently, it behaves as
#    follows:
#          missing entirely: default save (to ARCHIVE/YEAR.FOLDERNAME)
#          '-'             : do not generate save-hook
#          'ym'            : save to  ARCHIVE/YEAR.MONTH.FOLDERNAME
#          (other values to be added here in the future?)
#     IMPORTANT: to change the default save-hook, just edit accordingly
#     $SAVE_HOOK
#
# 4) Added support for groups of random addresses (ex students in one class)
#    Now, one entry like:
#
#     name:           myfamily
#     address:        me@here.com, wife@there.com, son@college.com
#     profile:        profile.family
#     recipe:
#        __GROUP__
#        __DESTINATION__
#     end:
#    should create a folder myfamily in which procmail will put all the messages
#    From: any of the listed addresses. CAUTION! Procmail recipe generation is
#    *not* one of my strongest skills, please triple-check that part (i.e. where
#    the __GROUP__ marker is used. Furthermore, send-hook directives binding
#    those addresses to the profile specified will be created
#
#  5) Added a "tohdr:" directive which adds a "folder-hook {'user'} my_hdr
#    To: {address'}" to muttrc. I.e. it allows automatic post to a mailing
#    list when in the mailing list folder. If "tohdr" is not present, no such
#    folder-hook is created.

#  6) Added creation of folders if not existing. This is mbox format only, and
#     must be enabled by setting $CREATE_FOLDERS to any numeric value > 0

#  7) Added loading of default profile (profile.default)
#
###########################################################################
# TODO:
#     add external rc file
#     find out what is the limit for line length in muttrc
#     find out what is the limit for line length in muttrc
#     allow custom name building (EX: folder names as YEAR.MONTH.NAME,
#             in whatever order
#     add mutt alias generation? Naah, that must happen somewhere else
#     add support for custom folder/send hooks (folder-hook: section similar to
#             the current recipe: one

use strict;
###########################################################################
# START OF CONFIGURATION PART (should become a separate rc file someday)
my $CONFIG               = "$ENV{'HOME'}/.mutt/procmailmuttrc";
my $CREATE_FOLDERS       = 0;
my $MAILDIR              = "$ENV{'HOME'}/.MAIL";
# to be sourced (in the right place?) in .muttrc
my $MUTTRULES_FOLDER     = "$ENV{'HOME'}/.mutt/mutt_folder_rules";

# this will have to be loaded in procmailrc with the includerc
# directive
my $PROCMAILRULES_FOLDER = "$ENV{'HOME'}/.procmail_folder_rules";

# Path where mutt profiles are stored
my $PROFILES_DIR         = "$ENV{'HOME'}/.mutt/profiles/";
my $PROFILES_DEFAULT 	 = "profile.default";
my @FOLDERS; # The array  containing all the parameters to create mutt and
             # procmailrc rules. It is an array to make sure that folders
             # are processed in the required order

my $PREFIX = "IN/"; # the prefix, relative to procmail's MAILDIR, and mutt's
                    # folder variable

# End of configuration part
###########################################################################
my $CURFOLDER; # The folder being currently processed
my $IDX;       # index of the @FOLDERS array
my $RECIPE_ON; # equal to 'Y' when loading a procmail recipe

# mutt requires some writing to temp variables and then reassembling
my $MUTT_RULES;
my $MUTT_MAILBOXES = "mailboxes ! ";

#####################################################################
#Load all configuration in the @FOLDERS array
open (CONFIG, "< $CONFIG") || die "Could not open $CONFIG;\n";

while (<CONFIG>) { 

    next if m/^#/; # comments allowed, of course

    if (($RECIPE_ON eq 'Y') && ($_ !~ m/^end:/)) {
	s/^\t*//; # remove indentation
	if ($_ =~ m/__DESTINATION__/) {
	    $FOLDERS[$IDX]{'recipe'} .= "${PREFIX}$FOLDERS[$IDX]{'name'}\n";
	}
	if ($_ =~ m/__GROUP__/) {
	    my $GROUP_RECIPE = ":0:\n* ^From: (";
	    $FOLDERS[$IDX]{'address'} =~ s/,\s*/,/g;
 	    my @GROUP = split /,/, $FOLDERS[$IDX]{'address'};
	    foreach my $MEMBER (@GROUP) {
		$MEMBER =~ s/^\s*//;
		$MEMBER =~ s/\s*$//;
		$GROUP_RECIPE .= "$MEMBER|";	
	    }
	    $GROUP_RECIPE =~ s/\|$//;
	    $GROUP_RECIPE .= ")\n";
	    $FOLDERS[$IDX]{'recipe'} .= $GROUP_RECIPE;
	}
	$FOLDERS[$IDX]{'recipe'} .=$_ if ($_ !~ m/(__DESTINATION__|__GROUP__)/);
    }

    if (m/^name:\t+(.*)$/) {
	$CURFOLDER = $1;
	print "# Loading $CURFOLDER;\n";
	$FOLDERS[$IDX]{'name'} = $CURFOLDER;
    }

    if (m/^(address|profile):\t+(.*)$/) {
	$FOLDERS[$IDX]{$1} = $2;
	}

    if (m/^lists:\t+(S|L)$/) {
	$FOLDERS[$IDX]{'lists'} = $1;
	}

    if (m/^tohdr:\t+(\S+)$/) {
	$FOLDERS[$IDX]{'tohdr'} = $1;
	}

    if (m/^save:\t+(\S+)$/) {
	$FOLDERS[$IDX]{'save'} = $1;
	}

    $RECIPE_ON = 'Y' if (m/^recipe:/);

    if (m/^end:/) {
	$RECIPE_ON = '';
	$IDX++; # on to the next folder
	}

}

################################################################################

#Now let's generate the procmail and mutt rules

unlink $MUTTRULES_FOLDER;
open (MUTT_FILE,"> $MUTTRULES_FOLDER") || die "Can't open $MUTTRULES_FOLDER;\n";

unlink $PROCMAILRULES_FOLDER;
open (PROCMAIL_FILE,"> $PROCMAILRULES_FOLDER") ||
    die "Can't open $PROCMAILRULES_FOLDER;\n";

# Set defaults
$MUTT_RULES .= "folder-hook . 'unmy_hdr to'\n\n";
#$MUTT_RULES .= "send-hook . 'source ${PROFILES_DIR}${PROFILES_DEFAULT}'\n\n";
$MUTT_RULES .= "folder-hook . 'source ${PROFILES_DIR}${PROFILES_DEFAULT}'\n\n";

my $i;          # just your ordinary loop variable

for ($i = 0; $i <= $#FOLDERS;$i++) {
    print PROCMAIL_FILE "$FOLDERS[$i]{'recipe'}\n\n";
    my $FOLD_PATH    = "=${PREFIX}$FOLDERS[$i]{'name'}";

    $MUTT_MAILBOXES .= " $FOLD_PATH ";
    $MUTT_RULES     .= "subscribe\t$FOLDERS[$i]{'address'}\n"
	               if ($FOLDERS[$i]{'lists'} eq 'S');
    $MUTT_RULES     .= "lists\t\t$FOLDERS[$i]{'address'}\n"
	               if ($FOLDERS[$i]{'lists'} eq 'L');

    ######################################################################	
    # send-hooks section

    my @ADDRESS_GROUP = split /,/, $FOLDERS[$i]{'address'};
    foreach my $CUR_ADDR (@ADDRESS_GROUP) {
    $MUTT_RULES .=
	"send-hook\t($CUR_ADDR)\$ 'source ${PROFILES_DIR}$FOLDERS[$i]{'profile'}'\n";
    }
    
    ######################################################################
    #save-hooks and profiles section
    my $SAVE_HOOK = "folder-hook\t$FOLD_PATH 'save-hook ~A +ARCHIVE/`/bin/date +%Y`.$FOLDERS[$i]{'name'}'\n";

    $SAVE_HOOK = '' if ($FOLDERS[$i]{'save'} eq '-');
    $SAVE_HOOK = "folder-hook\t$FOLD_PATH 'save-hook ~A +ARCHIVE/`/bin/date +%Y.%m`.$FOLDERS[$i]{'name'}'\n"
	            if ($FOLDERS[$i]{'save'} eq 'ym');

    $MUTT_RULES .=
	$SAVE_HOOK.
	"folder-hook\t$FOLD_PATH 'source ${PROFILES_DIR}$FOLDERS[$i]{'profile'}'\n";
    $MUTT_RULES .= "folder-hook\t\=$FOLDERS[$i]{'name'} 'my_hdr To: $FOLDERS[$i]{'address'}'\n" if ($FOLDERS[$i]{'tohdr'} eq 'Y');
    $MUTT_RULES .= "\n";
    unless (($CREATE_FOLDERS == 0) ||
	   (-e "$MAILDIR/$FOLDERS[$i]{'name'}")
	    ){
        system "touch $MAILDIR/$FOLDERS[$i]{'name'}";
        chmod 0644, "$MAILDIR/$FOLDERS[$i]{'name'}";
    }
 }

print MUTT_FILE <<END_MUTTCONF;
$MUTT_MAILBOXES

$MUTT_RULES

END_MUTTCONF

    close PROCMAIL_FILE;
    close MUTT_FILE;
